#ifdef VC2005EE
#include "stdafx.h"
#endif

#ifdef BCPP50
#include <math.h>
#endif

// wywolanie w  C 
#define _USE_MATH_DEFINES
#include <math.h>

#include "funs.h"

/* blok transmitancji (1+sT1)/(1+sT2) z ograniczeniami, jak przy realizacji
	na wzmacniaczu operacyjnym z ograniczeniem napiecia w sprzezeniu zwrotnym:

         Us <------------
                     _
                __ /_
           |---|__/__|---| ograniczenie Usmin i Usmax (np. idealne diody Zenera)
           |    -/       |
           | R ___  C||  |
           |--|___|--||--| k = R2/R1, T = (R+R2)C, Tc = RC - w zasadzie moznaby
           |   ___   ||  |                                   wprowadzac zamiast
           |--|___|------|                                   R1, R2, R, C
   R1 ___  |        R2   |
  ---|___|-|-|\          | ograniczenie Ucc i Uss
            -| \         |
 x       ----| /---------|
         |  +|/     y
        /                                  *******************************/
void transmit_3(
	double &y,//wyjscie
   double x, //wejscie
   double R1, double R2, double R, double C,
   double &uc,
   double h // krok calkowania (step)
   ){

//ograniczenia Us
double const Usmin = -5.0;
double const Usmax =  5.0;
//ograniczenia wyjscia
double const Uss   = -5.0;
double const Ucc   =  5.0;

double T = (R+R2)*C;
double Tc = R*C;
double k = R2/R1;

double uc_ = T/(T+h)*uc + h*k/(h+T)*x; //hipoteza uc(n+1)
double us  = Tc*k/T*x + (1-Tc/T)*uc_;  //wg schematu powyzej
	if (us < Usmin) uc = Tc/(Tc+h)*uc + h/(Tc+h)*Usmin;
   else
   if (us > Usmax) uc = Tc/(Tc+h)*uc + h/(Tc+h)*Usmax;
   else
   uc = uc_;

   y  = -k*Tc/T*x - (1-Tc/T)*uc;
   if (y < Uss) y = Uss;
   else
   if (y > Ucc) y = Ucc;
};


//blok obliczajacy czas pomiedzy impulsami o stromosci dx0/dt >= xm/krok
//li 	- chwila ostatniego impulsu,
//xn	- wartosc sygnalu w chwili "poprzedniej"
void oblicz_interwal(
	double &y0,
   double &xn,
   double &li,
   double xm, double x0,
   double time) {

   if ((x0-xn) >= xm) {
   	y0 = time - li;
      li = time;
      };
   xn = x0;
};

//blok probkujaco-pamietajacy, strobowany zboczem stromosci >= XM/krok
// x 		 - sygnal wejsciowy
// STROB  - sygnal strobujacy
// STROBn - sygnal strobujacy w chwili "poprzedniej"
// yn  	 - ostatnia wartosc wyjscia
void oblicz_upp(
	double &y0, double x,
   double STROB, double &STROBn, double XM, double &yn)
   {

   if ((STROB-STROBn) >= XM) {
   	y0 = x;
      yn = y0;
      } else y0 = yn;
   STROBn = STROB;
};

//blok obliczajacy czas pomiedzy impulsami o stromosci |dx0/dt| >= xm/krok
//li 	- chwila ostatniego impulsu,
//xn	- wartosc sygnalu w chwili "poprzedniej"
void oblicz_interwal_sprytniej(
	double &y0,
   double &xn,
   double &li,
   double xm, double x0,
   double time,
   double factor) {

   if (fabs(x0-xn) >= xm) {
   	y0 = factor*(time - li);
      li = time;
      };
   xn = x0;
};


// Sterowanie silnikiem SRM Ns/Nr
// w procedurze wyznacza sie Ns impulsow, w czasie trwania ktorych powinny
// byc utrzymywane prady w poszczegolnych Ns-fazach stojana.
// y0 jest ustawiane "Ns-bitowo"
void sterowanie_SRM_N(
	double &y0, double Ns, double Nr,
	double predkosc, double kat_poczatkowy, double kat_trwania,
   double step,
	double &kat) {

//const Ns = 6;  // liczba biegunow stojana
//const Nr = 4;	// liczba biegunow wirnika
unsigned int impuls;
double kat_p, kat_p0;
double kat_fazy;// kat polozenia wirnika widziany od strony poszczegolnych faz
int i,j;
double const bit[6] = {1,2,4,8,16,32};

   y0 = 0;
   /**/
   kat += predkosc*step;
   for (i=0;i<int(Ns/2.0);i++) {
   	//kat_fazy = fmod(kat + i*4.0*M_PI/Ns, 2*M_PI);
      kat_fazy = fmod(kat + i*2.0*M_PI/Ns, 2*M_PI);
   	impuls = false;
   	for (j=0;j<Nr;j++) {
   		kat_p0 = kat_poczatkowy + j*2.0*M_PI/Nr;
   		if (kat_poczatkowy<0) kat_p = kat_p0+2.0*M_PI;
   			else
   			if (kat_poczatkowy<(2.0*M_PI)) kat_p = kat_p0;
   				else
   				kat_p = kat_p0-2.0*M_PI;

      	if (!impuls) {
   			if ((kat_p+kat_trwania) <= (2.0*M_PI)) {
   				impuls = (kat_fazy >= kat_p) &&
         			 	 	(kat_fazy <= (kat_p+kat_trwania));
               } else {
            	impuls = (kat_fazy >= kat_p) ||
            			   (kat_fazy <= (kat_p+kat_trwania-(2.0*M_PI)));
            	};
            };
         };
      if (impuls) y0 += bit[i];
      };
   /**/
};

// Sterowanie silnikiem SRM 6/4
// w procedurze wyznacza sie impulsy, w czasie trwania ktorych powinny
// byc utrzymywane prady w poszczegolnych (trzech) fazach stojana
void sterowanie_SRM(
	double &y0, double &y1, double &y2,
	double predkosc, double kat_poczatkowy, double kat_trwania,
   double step,
	double &YOL, double &YOH,
	double &kat) {

double const Ns = 6;  // liczba biegunow stojana
double const Nr = 4;	// liczba biegunow wirnika
unsigned int impuls;
double kat_p, kat_p0;
double kat_fazy;// kat polozenia wirnika widziany od strony poszczegolnych faz
int i,j;

   y0 = y1 = y2 = 0;

   kat += predkosc*step;
   for (i=0;i<3;i++) {
   	kat_fazy = fmod(kat + i*2*M_PI/(Ns/2), 2*M_PI);
   	impuls = false;
   	for (j=0;j<Nr;j++) {
   		kat_p0 = kat_poczatkowy + j*2*M_PI/Nr;
   		if (kat_poczatkowy<0) kat_p = kat_p0+2*M_PI;
   			else
   			if (kat_poczatkowy<2*M_PI) kat_p = kat_p0;
   				else
   				kat_p = kat_p0-2*M_PI;

      	if (!impuls) {
   			if ((kat_p+kat_trwania) <= 2*M_PI) {
   				impuls = (kat_fazy >= kat_p) &&
         			 	 	(kat_fazy <= (kat_p+kat_trwania));
               } else {
            	impuls = (kat_fazy >= kat_p) ||
            			   (kat_fazy <= (kat_p+kat_trwania-2*M_PI));
            	};
            };
         };
      switch (i) {
      	case 0:y0 = impuls ? YOH : YOL; break;
         case 1:y2 = impuls ? YOH : YOL; break;
         case 2:y1 = impuls ? YOH : YOL; break;
         };
      };
};

/* Histerezowe sterowanie tranzystorami (sh i sl) galezi falownika dla SRM wg
	zadania "code".
   Jezeli code jest niski, to oba tranzystory sa wylaczone, jezeli nie, to sh
   jest wlaczony a sl utrzymuje prad w zadanym obszarze
*/
void SRM_inverter_switch(
   double &sh, double &sl, double &sl_old,
	int code, int number,
   double current, double reference, double half_delta,
   double y0l, double y0h) {
int bit[6] = {1,2,4,8,16,32};

	sh = sl = y0l;
    if (code & bit[number-1]) {
   	sh = y0h;
      if (current > (reference + half_delta)) sl = y0l; else
      if (current < (reference - half_delta)) sl = y0h; else
      sl = sl_old;
   	};
   sl_old = sl;
};

/*
multiwibrator monostabilny o nastepujacych cechach:
	1. wzbudzany zboczem narastajacym "input" takim, ze d(input)/dt>3/h,
		przy czym poprzednia wartosc "input" jest zapamietywana jako "input_",
   2. czas trwania impusu ustawiany jako Ti[s],
	3. poziom niski ma wartosc "YL", wysoki "YH",
	4. ostatnia chwila wzbudzenia jest zapamietywana jako "x"
*/
void multi_mono(
	double &output,
	double input,
   double time,
	double Ti, double &input_, double YL, double YH,
	double &x) {

   if ( (x <= time) && ((x + Ti) >= time) ) {
   	//trwa wczesniej wzbudzony impuls
      output = YH;
      } else {
      // sprawdzenie zbocza
      if ((input - input_) > 3) {
      	// jest narastajace -> wzbudzenie
         x = time;
         output = YH;
         } else
         //nie ma impulsu
         output = YL;
      };
   input_ = input;
};

/*
detektor maksimum
	x0 - sygnal wejsciowy
	x0_  - poprzednia wartosc x0;
	dx0 - zmiana x0
	dx0_ - poprzednia wartosc dx0
	output_ - ostatnia wartosc maksimum
	noise - wartosc szumu
*/
void max_detect(
	double &output,
	double x0,
   double &dx0_, double &dx0, double &output_, double &noise,
	double &x0_) {

   if (noise==0) noise = 0.1;
   if (fabs(x0-x0_)>noise) {//szum
   	dx0 = (x0 - x0_);
      if ((dx0_ >= 0) && (dx0 <= 0)) output_ = x0_;
      x0_ = x0;
      dx0_ = dx0;
      };
   output = output_;
};

/*
detektor minimum
	x0 - sygnal wejsciowy
	x0_  - poprzednia wartosc x0;
	dx0 - zmiana x0
	dx0_ - poprzednia wartosc dx0
	a4 - ostatnia wartosc minimum
	noise - wartosc szumu
*/
void min_detect(
	double &output,
	double x0,
   double &dx0_, double &dx0, double &output_, double &noise,
	double &x0_) {

   if (noise==0) noise = 0.1;
   if (fabs(x0-x0_)>noise) {//szum
   	dx0 = (x0 - x0_);
      if ((dx0_ <= 0) && (dx0 >= 0)) output_ = x0_;
      x0_ = x0;
      dx0_ = dx0;
      };
   output = output_;
}

void mlevel_3_0(
   double am, double sn, double sm,
   double OFF, double ON,
   double& S14, double& S23) {
bool modul = sn > sm;
bool level = (sm > 0.5*am) || (sm < -0.5*am);

if (level) S14 = modul ? ON : OFF; else
if (sm > 0) S14 = OFF; else
S14 = ON;

if (sm > 0.5*am) S23 = ON; else
if (sm < -0.5*am) S23 = OFF; else
S23 = modul ? ON : OFF;

};
