
/* blok transmitancji (1+sT1)/(1+sT2) z ograniczeniami, jak przy realizacji
	na wzmacniaczu operacyjnym z ograniczeniem napiecia w sprzezeniu zwrotnym:

         Us <------------
                     _
                __ /_
           |---|__/__|---| ograniczenie Usmin i Usmax (np. idealne diody Zenera)
           |    -/       |
           | R ___  C||  |
           |--|___|--||--| k = R2/R1, T = (R+R2)C, Tc = RC - w zasadzie moznaby
           |   ___   ||  |                                   wprowadzac zamiast
           |--|___|------|                                   R1, R2, R, C
   R1 ___  |        R2   |
  ---|___|-|-|\          | ograniczenie Ucc i Uss
            -| \         |
 x       ----| /---------|
         |  +|/     y
        /                                  *******************************/
void transmit_3(
	double &y,//wyjscie
   double x, //wejscie
   double R1, double R2, double R, double C,
   double &uc,
   double h // krok calkowania (step)
   );

//blok obliczajacy czas pomiedzy impulsami o stromosci dx0/dt >= xm/krok
//li 	- chwila ostatniego impulsu,
//xn	- wartosc sygnalu w chwili "poprzedniej"
void oblicz_interwal(
	double &y0,
   double &xn,
   double &li,
   double xm, double x0,
   double time);

//blok obliczajacy czas pomiedzy impulsami o stromosci |dx0/dt| >= xm/krok
//li 	- chwila ostatniego impulsu,
//xn	- wartosc sygnalu w chwili "poprzedniej"
void oblicz_interwal_sprytniej(
	double &y0,
   double &xn,
   double &li,
   double xm, double x0,
   double time,
   double factor);

//blok probkujaco-pamietajacy, strobowany zboczem stromosci >= XM/krok
// x 		 - sygnal wejsciowy
// STROB  - sygnal strobujacy
// STROBn - sygnal strobujacy w chwili "poprzedniej"
// yn  	 - ostatnia wartosc wyjscia
void oblicz_upp(
	double &y0, double x,
   double STROB, double &STROBn, double XM, double &yn);

// Sterowanie silnikiem SRM Ns/Nr
// w procedurze wyznacza sie Ns impulsow, w czasie trwania ktorych powinny
// byc utrzymywane prady w poszczegolnych Ns-fazach stojana.
// y0 jest ustawiane "Ns-bitowo"
void sterowanie_SRM_N(
	double &y0, double Ns, double Nr,
	double predkosc, double kat_poczatkowy, double kat_trwania,
   double step,
	double &kat);

// Sterowanie silnikiem SRM 6/4
// w procedurze wyznacza sie impulsy, w czasie trwania ktorych powinny
// byc utrzymywane prady w poszczegolnych (trzech) fazach stojana
void sterowanie_SRM(
	double &y0, double &y1, double &y2,
	double predkosc, double kat_poczatkowy, double kat_trwania,
   double step,
	double &YOL, double &YOH,
	double &kat);

void SRM_inverter_switch(
   double &sh, double &sl, double &sl_old,
	int code, int number,
   double current, double reference, double half_delta,
   double y0l, double y0h);

/*
multiwibrator monostabilny o nastepujacych cechach:
	1. wzbudzany zboczem narastajacym "input" takim, ze d(input)/dt>3/h,
		przy czym poprzednia wartosc "input" jest zapamietywana jako "input_",
   2. czas trwania impusu ustawiany jako Ti[s],
	3. poziom niski ma wartosc "YL", wysoki "YH",
	4. ostatnia chwila wzbudzenia jest zapamietywana jako "x"
*/
void multi_mono(
	double &output,
	double input,
   double time,
	double Ti, double &input_, double YL, double YH,
	double &x);

/*
detektor maksimum
	x0 - sygnal wejsciowy
	x0_  - poprzednia wartosc x0;
	dx0 - zmiana x0
	dx0_ - poprzednia wartosc dx0
	output_ - ostatnia wartosc maksimum
	noise - wartosc szumu
*/
void max_detect(
	double &output,
	double x0,
   double &dx0_, double &dx0, double &output_, double &noise,
	double &x0_);

/*
detektor minimum
	x0 - sygnal wejsciowy
	x0_  - poprzednia wartosc x0;
	dx0 - zmiana x0
	dx0_ - poprzednia wartosc dx0
	a4 - ostatnia wartosc minimum
	noise - wartosc szumu
*/
void min_detect(
	double &output,
	double x0,
   double &dx0_, double &dx0, double &output_, double &noise,
	double &x0_);

void mlevel_3_0(
   double am, double sn, double sm,
   double OFF, double ON,
   double& S14, double& S23);   
