#ifndef samples_h
#define samples_h

#ifdef BCPP50
#include <classlib/arrays.h>
#endif
/**/
#ifdef BB50
#include <vcl.h>
#endif
/**/
/****** deklaracje dla tablicowania "okna" przebiegu ***************/

struct SampleRecord {double time, value;};
// Probki - wektor probek typu SampleRecord, wielkosci NumberOfSamples
// NumberOfSamples - niech krok probkowania jest h, interwal "okna"
//							pomiarowego - T, wtedy NumberOfSamples = T/h,
//							np 20ms/10us = 2k, tutaj "z gorka" daje 10k
#define NumberOfSamples 10000
// MinimalResolution - minimalna rozdzielczosc, umowna, byle >= krok obliczen
#define MinimalResolution 10e-6
//definicje rodzajow funkcji
#define ID_RMS 0
#define ID_AV 1
#define ID_FA 2
#define ID_FB 3
#define ID_FAB 2
/****** koniec deklaracji dla tablicowania "okna" przebiegu ********/

#ifdef BCPP50
class TMiernik {
#endif
#ifdef BB50
class TMiernik : public TObject {
#endif
#ifndef QT
public:
   __fastcall TMiernik(int id, double f_harm);
   virtual __fastcall ~TMiernik();
   double GetParameter(int mode);
	SampleRecord GetProbe(int index);
	int GetLastIndex();
   int GetID();
   void DopiszProbke(double v, double t, double Period);
   void SaveSamples();
   void RestoreSamples();
protected:
   SampleRecord *Probki;
   int IndeksOstatniej, IndeksOstatniej_;
   int IndeksPierwszej, IndeksPierwszej_;
   int IndeksPierwszejWCalce, IndeksPierwszejWCalce_;
   double calkaRMS, calkaRMS_;
   double calkaAV, calkaAV_;
   double calkaF_A, calkaF_B, calkaF_A_, calkaF_B_;
   double f;
   int IndeksNastepnej(int poprzednia);
   int IndeksPoprzedniej(int nastepna);
   int IndeksProbkiwgChwili(double time);
   bool JestStarsza(int badana, int indeks);
   void OperacjaNaCalce(
   	double &calka,
   	double v, double h, int tryb, double wspnik);
   void ObliczCalki(double Period);
	bool tablica_probek(
   	int &nr,
   	double v,
        double t,
   	double ResTime,
   	double Period);
   int ID;
   };
#endif

#ifdef BCPP50
void DeleteSamples();
void SaveSamples();
void RestoreSamples();

bool DopiszProbkeDoMiernika( //ustawia true jezeli pobrano probke
   int &nr,
   double v,
   double t,
   double ResTime,	// w USER a2
   double f_harm,		// w USER a3
   double Period		// w USER x3
   );

double GetParameter(int id, int mode);
#endif


#ifdef BB50
void DeleteSamples(TObjectList* &mierniki);
void SaveSamples(TObjectList* mierniki);
void RestoreSamples(TObjectList* mierniki);

bool DopiszProbkeDoMiernika( //ustawia true jezeli pobrano probke
   int &nr,
   double v,
   double t,
   double ResTime,	// w USER a2
   double f_harm,		// w USER a3
   double Period,		// w USER x3
   TObjectList *mierniki
   );

double GetParameter(TObjectList *mierniki, int id, int mode);
#endif

#endif
